// cristinel ababei
// this is a simple Arduino project which uses an HC-06 module
// to send wirelessly, via Bluetooth, a simple message repeatedly
// to an App on an Adroid phone or to my host Laptop (must have Bluetooth 
// connectivity enabled)
// in addition, in the beginning only, it also changes the name
// of the HC-06 module to "CRIS-HC06" to make it unique to my situation; this
// helps in a class-room scenario where multiple modules may be on
// at the same time;

#include <SoftwareSerial.h>

#define DEBUG true

// connect the HC-06 TX to Arduino pin 2 RX;
// connect the HC-06 RX to Arduino pin 3 TX through a voltage divider created
// with three identical resistors;
SoftwareSerial hc06_serial(2, 3); // RX | TX

void setup()
{
  // (1) after the Arduino is programmed, we can use the Arduino's serial
  // terminal as a debugging interface; for example, we can print to it
  // anything that the Arduino sends to the bluetooth module;
  // in our case, we'll use it to only display the response from
  // the HC-06 module when we configure it to change its name;
  Serial.begin(9600);
  
  // (2) initialize serial link to HC-06; baudrate for communicating 
  // with the HC-06 is 9600;
  Serial.println("\nConnecting to the BT module via serial...");
  hc06_serial.begin(9600); 
  // send three AT commands to the HC-06 module; the third one is 
  // the one that changes its name to "CRIS-HC06"; here, replace
  // “CRIS-HC06” with you own unique identifier! 
  Serial.println("\nChanging the name of the BT module...");
  sendCommand("AT\r\n",3000,DEBUG); 
  delay(1000);
  sendCommand("AT+VERSION\r\n",3000,DEBUG); 
  delay(1000);
  sendCommand("AT+NAME=CRIS-HC06\r\n",3000,DEBUG);
  delay(1000);
  // send the header message, which will be send by the HC-06 module via 
  // bluetooth to my Android app (or to my Laptop);
  hc06_serial.println("Messages from Cris: ");
  delay(1000);
}

void loop() 
{
    // (1) send message to HC-06, which will transmit it via bluetooth,
    // to the App on my smartphone or to the Laptop;
    hc06_serial.println("Cristinel says hi!"); 

    // (2) insert 3 seconds delay;
    delay(3000); 
}

// Name: sendCommand
// Description: function used to send AT commands to HC-06.
// Params: 
//  command - the data/command to send; 
//  timeout - the time to wait for a response; 
//  debug - print to Serial window? (true = yes, false = no);
// Returns: response from the HC-06 bluetooth module (if there is a reponse);
String sendCommand(String command, const int timeout, boolean debug)
{
    String response = "";
           
    hc06_serial.print(command); // send the read character to the hc-06;
    
    long int time = millis();   
    while ( (time+timeout) > millis()) {
      while (hc06_serial.available()) {      
        // hc-06 module has data; so display its output to the serial window; 
        char c = hc06_serial.read(); // read the next character
        response+=c;
      }  
    }
    
    if (debug) {
      Serial.print(response);
      Serial.print("\n");
    }   
    return response;
}